﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 커스텀 슬라이더
    /// </summary>
    public class CustomSlider : Slider
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 자동 이동 여부 속성 - AutoMoveProperty

        /// <summary>
        /// 자동 이동 여부 속성
        /// </summary>
        public static readonly DependencyProperty AutoMoveProperty = DependencyProperty.Register
        (
            "AutoMove", 
            typeof(bool), 
            typeof(CustomSlider), 
            new FrameworkPropertyMetadata(false, AutoMovePropertyChangedCallback)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 디폴트 포인트 이동 이용 가능 여부
        /// </summary>
        private bool defaultIsMoveToPointEnabled;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 자동 이동 여부 - AutoMove

        /// <summary>
        /// 자동 이동 여부
        /// </summary>
        public bool AutoMove
        {
            get
            {
                return (bool)GetValue(AutoMoveProperty);
            }
            set
            {
                SetValue(AutoMoveProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 자동 이동 여부 속성 변경시 콜백 처리하기 - AutoMovePropertyChangedCallback(d, e)

        /// <summary>
        /// 자동 이동 여부 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void AutoMovePropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            CustomSlider customSlider = d as CustomSlider;
            
            if(customSlider != null)
            {
                if((bool)e.NewValue)
                {
                    customSlider.defaultIsMoveToPointEnabled = customSlider.IsMoveToPointEnabled;
                    customSlider.IsMoveToPointEnabled        = true;

                    customSlider.PreviewMouseMove += customSlider_PreviewMouseMove;
                }
                else
                {
                    customSlider.IsMoveToPointEnabled = customSlider.defaultIsMoveToPointEnabled;

                    customSlider.PreviewMouseMove -= customSlider_PreviewMouseMove;
                }
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 커스텀 슬라이더 PREVIEW 마우스 이동시 처리하기 - customSlider_PreviewMouseMove(sender, e)

        /// <summary>
        /// 커스텀 슬라이더 PREVIEW 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private static void customSlider_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            if(e.LeftButton == MouseButtonState.Pressed)
            {
                CustomSlider customSlider = sender as CustomSlider;

                Point point = e.GetPosition(customSlider);

                customSlider.Value = point.X / (customSlider.ActualWidth / customSlider.Maximum);
            }
        }

        #endregion
    }
}