﻿using System.Collections.ObjectModel;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 미디어 재생 리스트 윈도우
    /// </summary>
    public partial class MediaPlayListWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리스트 인덱스 변경시 이벤트 - ListIndexChanged

        /// <summary>
        /// 리스트 인덱스 변경시 이벤트
        /// </summary>
        public event ListIndexChangedHandler ListIndexChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Delegate
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리스트 인덱스 변경시 핸들러 - ListIndexChangedHandler

        /// <summary>
        /// 리스트 인덱스 변경시 핸들러
        /// </summary>
        /// <param name="index">인덱스</param>
        public delegate void ListIndexChangedHandler(int index);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 소스 - ItemsSource

        /// <summary>
        /// 항목 소스
        /// </summary>
        public ObservableCollection<MediaPlayListItem> ItemsSource
        {
            set
            {
                this.listBox.ItemsSource = value;
            }
        }

        #endregion
        #region 선택 인덱스 - SelectedIndex

        /// <summary>
        /// 선택 인덱스
        /// </summary>
        public int SelectedIndex
        {
            set
            {
                this.listBox.SelectedIndex = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MediaPlayListWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MediaPlayListWindow()
        {
            InitializeComponent();

            this.listBox.AllowDrop = true;

            MouseLeftButtonDown           += (s, e) => DragMove();
            this.closeButton.Click        += (s, e) => Close();
            this.listBox.MouseDoubleClick += listBox_MouseDoubleClick;
            this.listBox.DragOver         += listBox_DragOver;
            this.listBox.Drop             += listBox_Drop;
            this.listBox.KeyDown          += listBox_KeyDown;
            this.listBox.SelectionChanged += (s, e) => this.listBox.Focus();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 리스트 박스 마우스 더블 클릭시 처리하기 - listBox_MouseDoubleClick(sender, e)

        /// <summary>
        /// 리스트 박스 마우스 더블 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listBox_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            Visual visual = e.OriginalSource as Visual;

            while(visual != null)
            {
                if(visual is ListBoxItem)
                {
                    ListIndexChanged?.Invoke(listBox.SelectedIndex);

                    break;
                }

                visual = VisualTreeHelper.GetParent(visual) as Visual;
            }
        }

        #endregion
        #region 리스트 박스 드래그 OVER 처리하기 - listBox_DragOver(sender, e)

        /// <summary>
        /// 리스트 박스 드래그 OVER 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listBox_DragOver(object sender, DragEventArgs e)
        {
            if((e.AllowedEffects & DragDropEffects.Copy) == DragDropEffects.Copy)
            {
                e.Effects = DragDropEffects.Copy;
            }
            else
            {
                e.Effects = DragDropEffects.None;
            }

            e.Handled = true;
        }

        #endregion
        #region 리스트 박스 DROP 처리하기 - listBox_Drop(sender, e)

        /// <summary>
        /// 리스트 박스 DROP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listBox_Drop(object sender, DragEventArgs e)
        {
            string[] filePathArray = e.Data.GetData(DataFormats.FileDrop) as string[];

            foreach(string filePath in filePathArray)
            {
                FileInfo fileInfo = new FileInfo(filePath);

                ObservableCollection<MediaPlayListItem> collection = this.listBox.ItemsSource as ObservableCollection<MediaPlayListItem>;

                if(fileInfo.Extension.Equals(".wmv") || fileInfo.Extension.Equals(".avi") || fileInfo.Extension.Equals(".mp4"))
                {
                    collection.Add
                    (
                        new MediaPlayListItem()
                        {
                            FileName = fileInfo.Name,
                            FilePath = filePath
                        }
                    );
                }
            }

            if(this.listBox.Items.Count > 0 && this.listBox.SelectedIndex < 0)
            {
                this.listBox.SelectedIndex = 0;
            }

            e.Handled = true;
        }

        #endregion
        #region 리스트 박스 키 DOWN 처리하기 - listBox_KeyDown(sender, e)

        /// <summary>
        /// 리스트 박스 키 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listBox_KeyDown(object sender, KeyEventArgs e)
        {
            if(e.Key == Key.Delete)
            {
                if(this.listBox.SelectedItem != null)
                {
                    int index = this.listBox.SelectedIndex;

                    (this.listBox.ItemsSource as ObservableCollection<MediaPlayListItem>).RemoveAt(index);

                    if(index - 1 >= 0)
                    {
                        this.listBox.SelectedIndex = index - 1;
                    }
                    else if(this.listBox.Items.Count >= 1)
                    {
                        this.listBox.SelectedIndex = index;
                    }
                }
            }
        }

        #endregion
    }
}